<?php
/**
 * Image Annotator Basic Settings Form.
 */
function annotations_basic_settings_form($form, &$form_state) {

  // Select theme.
  $annoTheme = variable_get('img_annotator.anno_theme', 'basic');
  $themeOptions = array(
    'basic' => 'annotorious/css/annotorious.css',
    'dark' => 'annotorious/css/theme-dark/annotorious-dark.css',
  );

  $form['anno_theme'] = array(
    '#type' => 'radios',
    '#options' => $themeOptions,
    '#title' => 'Annotation Theme',
    '#default_value' => $annoTheme ? $annoTheme : 'basic',
  );

  // Javascript annotation prompts.
  $showJsAlerts = variable_get('img_annotator.show_js_alerts', 0);
  $form['show_js_alerts'] = array(
      '#type' => 'checkbox',
      '#prefix' => '<div><b>Prompt Javascript alerts while annotating images<b></div>',
      '#title' => 'Show Alerts',
      '#default_value' => $showJsAlerts ? $showJsAlerts : 0,
  );

  $form['submit_button'] = array(
      '#type' => 'submit',
      '#value' => t('Save')
  );

  return $form;
}



/*
 * Submition: annotations_basic_settings_form.
 */
function annotations_basic_settings_form_submit($form, &$form_state) {
  $submitted = $form_state['values'];

  variable_set('img_annotator.anno_theme', $submitted['anno_theme']);
  variable_set('img_annotator.show_js_alerts', $submitted['show_js_alerts']);

  drupal_set_message(t('Configurations saved successfully.'));
}


/**
 * Image Annotator Node Settings Form.
 */
function annotations_node_settings_form($form, &$form_state) {
  $defaults = variable_get('img_annotator.node_settings', array());
  $contentTypes = node_type_get_types();

  $contentTypesList = array();
  foreach ($contentTypes as $contentType) {
    $contentTypesList[$contentType->type] = $contentType->name;
  }

  $nodeTypeList = array();
  foreach ($contentTypes as $contentType) {
    $type_key = 'node_type_' . $contentType->type;

    if (empty($defaults[$type_key])) {
      $defaults[$type_key]['flag'] = array();
      $defaults[$type_key]['img_fields'] = array();
    }

    $form[$type_key] = array(
        '#type' => 'fieldset',
        '#title' => $contentType->name,
        '#tree' => TRUE,
        '#collapsible' => TRUE,
        '#collapsed' => $defaults[$type_key]['flag'] ? FALSE : TRUE
    );

    $form[$type_key]['flag'] = array(
        '#type' => 'checkbox',
        '#title' => 'Enable Image Annotation',
        '#default_value' => $defaults[$type_key]['flag'] ? $defaults[$type_key]['flag'] : ''
    );

    $form[$type_key]['img_fields'] = array(
        '#type' => 'textfield',
        '#element_validate' => array('_validate_img_fields'),
        '#title' => 'Image Fields (Machine Name)',
        '#description' => 'Enable annotations for above comma separated image fields.',
        // '#required' => TRUE,
        '#default_value' => $defaults[$type_key]['img_fields'] ? $defaults[$type_key]['img_fields'] : '',
        '#states' => array(
            'visible' => array(
                ':input[name="' . $type_key . '[flag]"]' => array(
                    'checked' => TRUE
                )
            )
        ),
    );

    $nodeTypeList[] = $type_key;
  }

  $form['node_type_list'] = array(
      '#type' => 'hidden',
      '#value' => $nodeTypeList
  );

  $form['submit_button'] = array(
      '#type' => 'submit',
      '#value' => t('Save')
  );

  return $form;
}

/*
 * Validation: Comma separated with no space within the element.
 */
function _validate_img_fields($element, &$form_state, $form) {
  $field_val = $element['#value'];
  $fieldParents = $element['#array_parents'];

  // If empty value passed with 'Enable/Flag' field true.
  $flagVal = $form_state['values'][$fieldParents[0]]['flag'];
  if (!$field_val && $flagVal) {
    $error_msg = "Machine name cannot be blank.";
    form_error($element, t($error_msg));
  }

  $new_arr = array();
  $entity_type = 'node';
  $bundle_name = str_replace("node_type_", "", $fieldParents[0]);
  $error_msg = "Invalid image instance %img_field.";
  $img_fields_arr = explode(',', $field_val);

  // Checking for user errors.
  foreach ($img_fields_arr as $img_field) {
    if ($str = trim($img_field)) {
      $imgInstance = field_info_instance($entity_type, $str, $bundle_name);
      $imgInstanceType = isset($imgInstance['widget']['type']) ? $imgInstance['widget']['type'] : FALSE;

      if ($imgInstanceType != 'image_image') {
        form_error($element, t($error_msg, array('%img_field' => $str)));
        break;
      }
      else {
        $new_arr[] = $str;
      }
    }
  }

  // Properly formatting the comma separated values.
  form_set_value($element, implode(', ', $new_arr), $form_state);
}

/*
 * Submition: annotations_node_settings_form.
 */
function annotations_node_settings_form_submit($form, &$form_state) {
  $settings = array();

  $submitted = $form_state['values'];
  $nodeTypeList = $submitted['node_type_list'];

  // Initiate new configs
  foreach ($nodeTypeList as $type_key) {
    if ($submitted[$type_key]['flag']) {
      $settings[$type_key] = $submitted[$type_key];
    }
  }

  variable_set('img_annotator.node_settings', $settings);
  drupal_set_message(t('Configurations saved successfully.'));
}

/*
 * annotations_save_action
 */
function annotations_save_action() {
  global $user;

  $nid = isset($_POST['nid']) ? $_POST['nid'] : FALSE;
  $img_field = isset($_POST['img_field']) ? $_POST['img_field'] : FALSE;
  $annotation = isset($_POST['annotation']) ? $_POST['annotation'] : FALSE;

  $validParams = $nid && $img_field && $annotation;
  $canCreateAll = user_access('img_annotator create');
  $canCreateOwn = user_access('img_annotator own create');

  if ($validParams && $canCreateAll) {
    $table = 'img_annotator';
    $annotationEncoded = drupal_json_encode($annotation);

    $queryVals = array(
        'nid' => $nid,
        'uid' => $user->uid,
        'field' => $img_field,
        'annotation' => $annotationEncoded,
        'updated' => time(),
    );
    $aid = db_insert($table)->fields($queryVals)->execute();
    $response = $aid;
  }
  elseif ($validParams && $canCreateOwn) {
    $node = node_load($nid);
    if ($node->uid == $user->uid) {

      $table = 'img_annotator';
      $annotationEncoded = drupal_json_encode($annotation);

      $queryVals = array(
          'nid' => $nid,
          'uid' => $user->uid,
          'field' => $img_field,
          'annotation' => $annotationEncoded,
          'updated' => time()
      );
      $aid = db_insert($table)->fields($queryVals)->execute();
      $response = $aid;
    }
    else {
      $response = FALSE;
    }
  }
  else {
    $response = FALSE;
  }

  return drupal_json_output($response);
}

/*
 * annotations_update_action
 */
function annotations_update_action() {
  global $user;

  $annotation = isset($_POST['annotation']) ? $_POST['annotation'] : FALSE;
  $aid = isset($_POST['annotation']['aid']) ? $_POST['annotation']['aid'] : FALSE;
  $nid = isset($_POST['annotation']['nid']) ? $_POST['annotation']['nid'] : FALSE;

  $canEdit = user_access('img_annotator edit');
  $canEditOwn = user_access('img_annotator own edit');

  if ($annotation && $aid && $nid && $canEdit) {
    $table = 'img_annotator';
    $annotationEncoded = drupal_json_encode($annotation);

    $rowCount = db_update($table)
      ->fields(array('uid' => $user->uid, 'annotation' => $annotationEncoded))
      ->condition('aid', $aid)
      ->execute();

    $response = (bool) $rowCount;
  }
  elseif ($annotation && $aid && $nid && $canEditOwn) {

    $node = node_load($nid);
    if ($node->uid == $user->uid) {
      $table = 'img_annotator';
      $annotationEncoded = drupal_json_encode($annotation);

      $rowCount = db_update($table)
        ->fields(array('uid' => $user->uid, 'annotation' => $annotationEncoded))
        ->condition('aid', $aid)
        ->execute();

      $response = (bool) $rowCount;
    }
    else {
      $response = FALSE;
    }
  }
  else {
    $response = FALSE;
  }

  return drupal_json_output($response);
}

/*
 * annotations_delete_action
 */
function annotations_delete_action() {
  global $user;

  $annotation = isset($_POST['annotation']) ? $_POST['annotation'] : FALSE;
  $aid = isset($_POST['annotation']['aid']) ? $_POST['annotation']['aid'] : FALSE;
  $nid = isset($_POST['annotation']['nid']) ? $_POST['annotation']['nid'] : FALSE;

  $canDelete = user_access('img_annotator edit');
  $canDeleteOwn = user_access('img_annotator own edit');

  if ($aid && $nid && $canDelete) {

    $table = 'img_annotator';
    $deleted_count = db_delete($table)
      ->condition('aid', $aid)
      ->execute();

    $response = (bool) $deleted_count;
  }
  elseif ($aid && $nid && $canDeleteOwn) {

    $node = node_load($nid);
    if ($node->uid == $user->uid) {

      $table = 'img_annotator';
      $deleted_count = db_delete($table)
        ->condition('aid', $aid)
        ->execute();

      $response = (bool) $deleted_count;
    }
    else {
      $response = FALSE;
    }
  }
  else {
    $response = FALSE;
  }

  return drupal_json_output($response);
}

/*
 * annotations_retrieve_action
 */
function annotations_retrieve_action() {
  global $user;
  $nid = isset($_POST['nid']) ? $_POST['nid'] : FALSE;

  $node = node_load($nid);

  $canEdit = user_access('img_annotator edit');
  $canEditOwn = user_access('img_annotator own edit');
  $canRetrieve = $canEdit || user_access('img_annotator create') || user_access('img_annotator view');
  $canRetrieveOwn = $canEditOwn || user_access('img_annotator own create') || user_access('img_annotator own view');

  $nodeOwner = ($node->uid == $user->uid) ? TRUE : FALSE;

  if ($nid && $canRetrieve) {
    $anno_editable = TRUE;
    $table = 'img_annotator';
    $annotation_arr = array();

    // Existing Annotation from table.
    $existing_annotation = db_select($table, 'i')
      ->fields('i', array('aid', 'annotation'))
      ->condition('nid', $nid)
      ->execute();

    // User can edit
    if (!$canEdit && !($canEditOwn && $nodeOwner)) {
      $anno_editable = FALSE;
    }

    // User can not view.
    // It does not load library coz of code at .module alteration.

    // Prepare annotation response
    while ($record = $existing_annotation->fetchObject()) {
      $anno_val = drupal_json_decode($record->annotation);
      $anno_val['editable'] = $anno_editable;
      $anno_val['aid'] = $record->aid;

      $annotation_arr[] = $anno_val;
    }

    $response = drupal_json_encode($annotation_arr);
  }
  elseif ($nid && $canRetrieveOwn) {

    if ($nodeOwner) {
      $anno_editable = TRUE;
      $table = 'img_annotator';
      $annotation_arr = array();

      // Existing Annotation from table
      $existing_annotation = db_select($table, 'i')
        ->fields('i', array('aid', 'annotation'))
        ->condition('nid', $nid)
        ->execute();

      // User can edit
      if (!$canEdit && !($canEditOwn && $nodeOwner)) {
        $anno_editable = FALSE;
      }

      // User can not view
      // It does not load library coz of code at .module alteration

      // Prepare annotation response
      while ($record = $existing_annotation->fetchObject()) {
        $anno_val = drupal_json_decode($record->annotation);
        $anno_val['editable'] = $anno_editable;
        $anno_val['aid'] = $record->aid;

        $annotation_arr[] = $anno_val;
      }

      $response = drupal_json_encode($annotation_arr);
    }
    else {
      $response = FALSE;
    }
  }
  else {
    $response = FALSE;
  }

  return drupal_json_output($response);
}